(ns core
  (:require [cljs.nodejs :as node]
            [xmlhttprequest :refer [XMLHttpRequest]]
            [targets :as targets]
            [users :as users]
            [templates :as templates]
            [login :as login]
            [tasks :as tasks]
            [agent :as agent]
            [tasksrepl :as tasksrepl]
            [version :as version]))

(def program (node/require "commander"))

;;; Signup
(defn signup-commands [program]
  (let [signup (-> program (.command "signup"))]
    (doto signup
      (.description "Signup")
      (.action login/signup))))

;;; Login
(defn login-commands [program]
  (let [login (-> program (.command "login"))]
    (doto login
      (.description "Login")
      (.action login/login))))

;;; Logout
(defn logout-commands [program]
  (let [logout (-> program (.command "logout"))]
    (doto logout
      (.description "logout")
      (.action login/logout))))

;;; Templates
(defn templates-commands [program]
  (let [templates (-> program (.command "templates"))
        get       (.command templates "get")
        list      (.command templates "list")
        run       (.command templates "run")]
    (doto list
      (.description "List Templates")
      (.action templates/list-templates))

    (doto get
      (.description "Get Template Params")
      (.requiredOption "-n --name <name>" "The name of the Template")
      (.action templates/get-template))

    (doto run
      (.description "Create a Task using a Template")
      (.requiredOption "-n --name <name>" "The name of the Template")
      (.option "-m --message <message>" "A message with the description of the task")
      (.requiredOption "-t --target <target>" "Target where the task should run (ex: db-prod, k8s-homolog)")
      (.option "-y --type <type>" "The type of the script (ex: postgres, python, k8s) - overrides the Target type")
      (.option "-p --params <params>"
               "A comma-separated list with the parameters required in the Template, in the format: `param_name=param_value`")
      (.action templates/run-template))))

;;; Targets
(defn targets-commands [program]
  (let [targets (-> program (.command "targets"))
        create    (.command targets "create")
        update    (.command targets "update")
        list    (.command targets "list")
        get    (.command targets "get")]
    (doto list
      (.description "List Targets")
      (.action targets/list-targets))

    (doto get
      (.description "Get Target")
      (.requiredOption "-n --name <name>" "The name of the target")
      (.action targets/get-target))

    (doto create
      (.description "Create a new Target. Check out https://docs.runops.io")
      (.requiredOption "--name <name>"
                       "The name used to create Tasks with this Target")
      (.requiredOption "--type <type>"
                       "The type of the Target")
      (.option "--message <message>"
               "A message to help users identify the Target")
      (.option "--runner <runner>"
               "The type of runner for the Target: `runops-hosted` or `self-hosted`", "runops-hosted")
      (.option "--review <review>"
               "`none`: no reviews. `default`: the settings of the org. `teams`: the list of teams in the `reviewers` option")
      (.option "--channel <channel>"
               "The name of the Slack channel to send reviews")
      (.option "--reviewers <reviewers>"
               "A comma-separated list of names with the Teams that must review Tasks on this Target.")
      (.option "--secrets <secrets>"
               "A comma-separated list with the secrets to attach to the Target, in the format: `SECRET_NAME=SECRET_VALUE`")
      (.option "--runner_provider <message>"
               "The runner provider: `runops` or `github`. Defaults to `runops` if not provided")
      (.option "--secret_provider <message>"
               "The secret manager provider: `aws`, `hashicorp/db`, `hashicorp/kv` or `env-var`. Defaults to null if not provided")
      (.option "--secret_path <message>"
               "The secret name or path in the provider")
      (.option "--secret_mapping <json-key-value>"
               "A JSON structure to config the secret mapping")
      (.option "--tags <message>"
               "For `runops` runner provider only, it identifies the environment. Any string like `dev`, `prod`, etc")
      (.option "--groups <message>"
               "A comma separated string with all allowed user groups that can read and use the target")
      (.option "--config <message>"
               "A JSON structure to config target secrets in clear text (dev mode)")
      (.option "--redact <message>"
               "If the target will have its tasks redacted: ['all', 'none']. Defaults do 'all' if not provided")
      (.action targets/create))

    (doto update
      (.description "Update a Target. Check out https://docs.runops.io")
      (.requiredOption "--name <name>"
                       "The name of the Target to update")
      (.option "--type <type>"
               "The type of the Target")
      (.option "--message <message>"
               "A message to help users identify the Target")
      (.option "--runner <runner>"
               "The type of runner for the Target: `runops-hosted` or `self-hosted`")
      (.option "--review <review>"
               "`none`: no reviews. `default`: the settings of the org. `teams`: the list of teams in the `reviewers` option")
      (.option "--channel <channel>"
               "The name of the Slack channel to send reviews")
      (.option "--reviewers <reviewers>"
               "A comma-separated list of names with the Teams that must review Tasks on this Target")
      (.option "--secrets <secrets>"
               "A comma-separated list with the secrets to attach to the Target, in the format: `SECRET_NAME=SECRET_VALUE`")
      (.option "--runner_provider <message>"
               "The runner provider: `runops` or `github`. Defaults to `runops` if not provided")
      (.option "--secret_provider <message>"
               "The secret manager provider: `aws`, `hashicorp/db`, `hashicorp/kv` or `env-var`. Defaults to null if not provided")
      (.option "--secret_path <message>"
               "The secret name or path in the provider")
      (.option "--secret_mapping <json-key-value>"
               "A JSON structure to config the secret mapping")
      (.option "--tags <message>"
               "For `runops` runner provider only, it identifies the environment. Any string like `dev`, `prod`, etc")
      (.option "--groups <message>"
               "A comma separated string with all allowed user groups that can read and use the target")
      (.option "--config <message>"
               "A JSON structure to config target secrets in clear text (dev mode)")
      (.option "--redact <message>"
               "If the target will have its tasks redacted: ['all', 'none']. Defaults do 'all' if not provided")
      (.option "--status <message>"
               "The status of the target: ['active', 'inactive']")
      (.action targets/update-cmd))))

;;; Tasks
(defn tasks-commands [program]
  (let [tasks (-> program (.command "tasks"))
        create (.command tasks "create")
        list (.command tasks "list")
        logs (.command tasks "logs")
        get (.command tasks "get")
        run (.command tasks "run")
        repl (.command tasks "repl")]

    (doto create
      (.description "Create a new Task. Check out https://docs.runops.io")
      (.option "-m --message <message>" "A message with the description of the task")
      (.requiredOption "-t --target <target>" "Target where the task should run 🚨-t is now the Target (previously -g)🚨")
      (.option "-y --type <type>" "The type of the script - overrides the Target type")
      (.option "-s --script <script>" "The contents of the script of the task")
      (.option "-f --file <file>" "The path to a local file with the script")
      (.action tasks/create-task))

    (doto list
      (.description "List tasks")
      (.action tasks/list-tasks))

    (doto get
      (.description "Get task details")
      (.requiredOption "-i --id <id>" "The id of the task")
      (.action tasks/get-task))

    (doto run
      (.description "Run a Task")
      (.requiredOption "-i --id <id>" "The id of the task")
      (.action tasks/run-task))

    (doto logs
      (.description "Get task logs")
      (.requiredOption "-i --id <id>" "The id of the task")
      (.action tasks/get-task-logs))

    (doto repl
      (.description "Start a REPL")
      (.action (fn [_] (tasksrepl/start-repl tasksrepl/welcome-message))))))

;;; Users
(defn users-commands [program]
  (let [users (-> program (.command "users"))
        create (.command users "create")
        update (.command users "update")
        list (.command users "list")]
    (doto list
      (.description "List Users")
      (.action users/list-users))

    (doto create
      (.description "Create a User")
      (.requiredOption "--name <name>"
                       "The name of the User")
      (.requiredOption "--email <email>"
                       "The email of the User")
      (.option "--status <status>"
               "The status of the User")
      (.option "--team <team>"
               "The team of the User")
      (.option "--groups <groups>"
               "A comma separated string with user groups")
      (.action users/create))

    (doto update
      (.description "Update a User")
      (.requiredOption "-i --id <id>"
                       "The id of the User to update")
      (.option "--status <status>"
               "The status of the User")
      (.option "--team <team>"
               "The team of the User")
      (.option "--groups <groups>"
               "A comma separated string with user groups")
      (.action users/update-cmd))))

(defn agent-commands [program]
  (let [agents (-> program
                   (.command "agents")
                   (.description "Manage agents"))
        create-token (.command agents "create-token")
        list (.command agents "list")
        dockerRun (.command agents "docker-run")]

    (doto list
      (.description "List Connected Agents")
      (.action agent/list-agents))

    (doto create-token
      (.description "Create an agent token. [WARNING] if you have agents running, this operation will break then!")
      (.option "-f --force"
               "Don't prompt when creating a new token")
      (.action agent/create-agent-token))

    (doto dockerRun
      (.description "Runs an agent locally.")
      (.requiredOption "--token <token>" "The agent token.")
      (.option "--tags <tags>" "The tag in which this agent will start.")
      (.option "--dryrun" "Only output to stdout the command that will be executed.")
      (.option "--envvar <envvar>" "The name of the env-var which are going to be used in the agent" "ENV_CONFIG")
      (.option "--ev <env...>" "Set environment variables for the envvar option, could be specified multiple times.")
      ;; docker related options
      (.option "-e --env <env...>" "Set environment variables which are going to be mapped to container, could be specified multiple times.")
      (.option "--name <name>" "Assign a name to the container.")
      (.option "--rm" "Automatically remove the container when it exits.")
      (.action agent/docker-run))))

(defn print-version []
  (if (true? (.-version (.opts program)))
    (do (println (version/get-current-version))
        (version/print-update-version-message))
    (.help program)))

(let [_ (tasks-commands program)
      _ (targets-commands program)
      _ (templates-commands program)
      _ (login-commands program)
      _ (logout-commands program)
      _ (signup-commands program)
      _ (users-commands program)
      _ (agent-commands program)])

(defn start [& _]
  (doto program
    (.option "-v --version" "show cli version")
    (.action print-version)
    (.parse (.-argv node/process))))

(node/enable-util-print!)
(set! js/XMLHttpRequest XMLHttpRequest)
(set! *main-cli-fn* start)

(defn isdev-environment []
  (= ^js (.-RUNOPS_ENV (.-env node/process)) "development"))

(defn main []
  (if (isdev-environment)
    ()
    (start)))
