(ns tasks
  (:require [cljs.nodejs :as node]
            [cljs.core.async :refer [<! go]]
            [client :as client]
            [traces :as traces]
            [auth :as auth]
            [spinner :as spinner]
            [printer :as printer]
            [version :as version]))

(defmulti tasks-values (fn [data] (map? data)))

(defmethod tasks-values false [data]
  (->> data
       (map #(vals (select-keys % [:id :status :created :description])))))

(defmethod tasks-values true [data]
  data)

(defn parse-task-no-logs [task result]
  (some-> task
          tasks-values
          (#(conj {} {:content %
                      :outcome (:outcome result)
                      :print-type (:print-type result)
                      :table-head (:table-head result)
                      :columns-width (:columns-width result)}))
          printer/print-data))

(defn extract-logs [task]
  (if (:task_logs task) (:task_logs task) task))

(defn parse-logs-response [task]
  (some-> task
          extract-logs
          clj->js
          js/console.log))

(defn parse-response [result]
  (go (when-let [response (<! (:response-chan result))]
        (some-> response
                spinner/stop
                client/extract-body
                (#(if (or (:task_logs %) (:logs_url %))
                    (parse-logs-response %)
                    (parse-task-no-logs % result)))
                traces/close))))

(defn node-slurp [path]
  (let [fs (node/require "fs")]
    (.readFileSync fs path "utf8")))

(defn read-script [task]
  (try (if (:script task)
         task
         (assoc task :script (node-slurp (:file task))))
       (catch js/Error e (js/console.log (.toString e)))))

;; create task
(defn create-task-api [task]
  (let [_ (spinner/start nil)
        response-chan (client/create-task task)]
    (assoc task :response-chan response-chan)))

(defn create-outcome [data]
  (assoc data :print-type :get
         :outcome "Task created successfully !!"))

(defn create-task [command]
  (version/print-update-version-message)
  (.withTrace traces/event (clj->js {:command "create-task"})
              #(some-> (conj {:target (.-target command)
                              :script (.-script command)
                              :file (.-file command)}
                             (when-let [message (.-message command)]
                               {:message message})
                             (when-let [task-type (.-type command)]
                               {:type task-type}))
                       read-script
                       auth/key-auth
                       create-task-api
                       create-outcome
                       parse-response)))

;; list tasks
(defn list-tasks-api [task]
  (let [_ (spinner/start nil)
        response-chan (client/list-tasks task)]
    (assoc task :response-chan response-chan)))

(defn list-outcome [data]
  (assoc data :print-type :list
         :outcome "List tasks"
         :table-head ["Id" "Status" "Created" "Message"]
         :columns-width [7, 17, 18, 38]))

(defn list-tasks []
  (version/print-update-version-message)
  (.withTrace traces/event
              (clj->js {:command "list-tasks"})
              #(some-> (auth/key-auth {})
                       list-tasks-api
                       list-outcome
                       parse-response)))

;; get task
(defn get-task-api [task]
  (let [_ (spinner/start nil)
        response-chan (client/get-task task)]
    (assoc task :response-chan response-chan)))

(defn get-outcome [data]
  (assoc data :print-type :get
         :outcome "Get task"))

(defn get-task [command]
  (version/print-update-version-message)
  (.withTrace traces/event
              (clj->js {:command "get-task"})
              #(some-> (auth/key-auth {:id (.-id command)})
                       get-task-api
                       get-outcome
                       parse-response)))

;; get task logs
(defn get-task-logs-api [task]
  (let [_ (spinner/start nil)
        response-chan (client/get-task-logs task)]
    (assoc task :response-chan response-chan)))

(defn get-logs-outcome [data]
  (assoc data :print-type :get
         :outcome "Get task"))

(defn get-task-logs [command]
  (.withTrace traces/event
              (clj->js {:command "get-task-logs"})
              #(some-> (auth/key-auth {:id (.-id command)})
                       get-task-logs-api
                       get-logs-outcome
                       parse-response)))

;; run task
(defn run-task-api [task]
  (let [_ (spinner/start nil)
        response-chan (client/run-task task)]
    (assoc task :response-chan response-chan)))

(defn run-outcome [data]
  (assoc data :print-type :get
         :outcome "Run task !"))

(defn run-task [command]
  (version/print-update-version-message)
  (.withTrace traces/event
              (clj->js {:command "run-task"})
              #(some-> (auth/key-auth {:id (.-id command)})
                       run-task-api
                       run-outcome
                       parse-response)))
